@@ -3,14 +3,16 @@
 # @param Functions list of functions
 # @param condition a single row from the design input
 # @param replications number of times to repeat the Monte Carlo simulations
-# @param fixed_design_elements optional object containing fixed conditions across design
+# @param fixed_objects optional object containing fixed conditions across design
 # @param cl cluster object defined from the parallel package
 # @param MPI logical; flag passed down from the runSimulation function
 # @param save_results logical; save the results to .rds files
 # @param results_filename the file name used to store results in
 #
-Analysis <- function(Functions, condition, replications, fixed_design_elements, cl, MPI, seed,
-                     save_results, results_filename)
+Analysis <- function(Functions, condition, replications, fixed_objects, cl, MPI, seed,
+                     save_results, save_results_dirname, max_errors,
+                     save_generate_data, save_generate_data_dirname,
+                     export_funs, packages)
 {
     # This defines the work-flow for the Monte Carlo simulation given the condition (row in Design)
     #  and number of replications desired
@@ -19,19 +21,26 @@ Analysis <- function(Functions, condition, replications, fixed_design_elements,
         cell_results <- lapply(1L:replications, mainsim, condition=condition,
                                generate=Functions$generate,
                                analyse=Functions$analyse,
-                               fixed_design_elements=fixed_design_elements)
+                               fixed_objects=fixed_objects,
+                               max_errors=max_errors, packages=packages,
+                               save_generate_data=save_generate_data,
+                               save_generate_data_dirname=save_generate_data_dirname)
     } else {
         if(MPI){
             i <- 1L
-            cell_results <- foreach(i=1L:replications) %dopar%
+            cell_results <- foreach(i=1L:replications, .export=export_funs) %dopar%
                 mainsim(i, condition=condition, generate=Functions$generate,
-                     analyse=Functions$analyse, fixed_design_elements=fixed_design_elements)
+                     analyse=Functions$analyse, fixed_objects=fixed_objects,
+                     max_errors=max_errors, save_generate_data=save_generate_data,
+                     save_generate_data_dirname=save_generate_data_dirname, packages=packages)
         } else {
             if(!is.null(seed)) parallel::clusterSetRNGStream(cl=cl, seed[condition$ID])
             cell_results <- parallel::parLapply(cl, 1L:replications, mainsim,
                                                 condition=condition, generate=Functions$generate,
                                                 analyse=Functions$analyse,
-                                                fixed_design_elements=fixed_design_elements)
+                                                fixed_objects=fixed_objects, packages=packages,
+                                                max_errors=max_errors, save_generate_data=save_generate_data,
+                                                save_generate_data_dirname=save_generate_data_dirname)
         }
     }
 
@@ -53,14 +62,15 @@ Analysis <- function(Functions, condition, replications, fixed_design_elements,
             message(sprintf('WARNING: analyse() returned NA or NaN values from row %i in Design',
                             condition$ID))
     }
-    if(save_results)
-        saveRDS(list(condition=condition, results=results, try_errors=try_errors),
-                paste0('SimDesign_results/', results_filename, 'ROWID-', condition$ID, '.rds'))
+    if(save_results){
+        tmpfilename <- paste0(save_results_dirname, '/results-row-', condition$ID, '.rds')
+        saveRDS(list(condition=condition, results=results, errors=try_errors), tmpfilename)
+    }
     sim_results <- Functions$summarise(results=results, parameters_list=parameters,
-                           condition=condition, fixed_design_elements=fixed_design_elements)
+                           condition=condition, fixed_objects=fixed_objects)
 
     if(!is.vector(sim_results) || is.null(names(sim_results)))
         stop('summarise() must return a named vector', call.=FALSE)
-    sim_results <- c(sim_results, 'TRY_ERROR_MESSAGE: '=try_errors)
+    sim_results <- c(sim_results, 'REPLICATIONS'=replications, 'ERROR_MESSAGE: '=try_errors)
     return(sim_results)
 }
